@@ -58,6 +58,7 @@ static char *staticligkern[] = {
 	"% LIGKERN nine {} * ; * {} nine ;",
 
 /* Kern accented characters the same way as their base. */
+/* There cannot be used currently and what are they for, anyway?
 	"% LIGKERN Aacute <> A ; aacute <> a ;",
 	"% LIGKERN Acircumflex <> A ; acircumflex <> a ;",
 	"% LIGKERN Adieresis <> A ; adieresis <> a ;",
@@ -88,7 +89,7 @@ static char *staticligkern[] = {
 	"% LIGKERN Yacute <> Y ; yacute <> y ;",
 	"% LIGKERN Ydieresis <> Y ; ydieresis <> y ;",
 	"% LIGKERN Zcaron <> Z ; zcaron <> z ;",
-
+*/
 /*
  *   These next are only included for deficient afm files that
  *   have the lig characters but not the lig commands.
@@ -561,7 +562,7 @@ int readprotusion(PSDoc *psdoc, PSFont *psfont, const char *filename) {
 #endif
 
 	metrics = psfont->metrics;
-	if(NULL == (fp = fopen(filename, "r"))) {
+	if(NULL == (fp = ps_open_file_in_path(psdoc, filename))) {
 		return(-1);
 	}
 
@@ -593,7 +594,7 @@ ADOBEFONTMETRIC *readadobe(PSDoc *psdoc, const char *filename) {
 	}
 	memset(metric, 0, sizeof(ADOBEFONTMETRIC));
 
-	if(0 == (metric->afmin = fopen(filename, "r"))) {
+	if(0 == (metric->afmin = ps_open_file_in_path(psdoc, filename))) {
 		ps_error(psdoc, PS_RuntimeError, _("Couldn't open afm file: %s\n"), filename);
 		return(NULL);
 	}
@@ -855,6 +856,10 @@ static void checkligkern(PSDoc *psdoc, ADOBEFONTMETRIC *metrics, char *s)
 			if (n == 3 && strcmp(mlist[1], "{}") == 0) { /* rmkern command */
 				rmkern(psdoc, gadobechars, mlist[0], mlist[2], (ADOBEINFO *)0) ;
 			} else if (n == 3 && strcmp(mlist[1], "<>") == 0) { /* addkern */
+				/* FIXME: What is the meaning of kern_equivs?
+				 * addkern() does not set the kern by kern_equivs which is not
+				 * used.
+				 */
 				addkern(psdoc, gadobechars, mlist[0], mlist[2]) ;
 			} else if (n == 3 && strcmp(mlist[0], "||") == 0 &&
 										strcmp(mlist[1], "=") == 0) { /* bc command */
@@ -1024,7 +1029,7 @@ int readencoding(PSDoc *psdoc, ADOBEFONTMETRIC *metrics, const char *enc) {
 		return -1;
 	}
 	if(enc) {
-		metrics->afmin = fopen(enc, "r") ;
+		metrics->afmin = ps_open_file_in_path(psdoc, enc) ;
 		param = 0 ;
 		if (metrics->afmin == NULL) {
 			ps_error(psdoc, PS_RuntimeError, _("Could not open encoding file '%s'."), enc);
