@@ -18,9 +18,11 @@
  *    Boston, MA 02111-1307, USA.
  */
 
-/* $Id: ps_util.c,v 1.24 2004/06/07 12:00:06 steinm Exp $ */
+/* $Id: ps_util.c,v 1.33 2004/07/21 11:07:50 steinm Exp $ */
 
 #include <stdio.h>
+#include <errno.h>
+#include <stdlib.h>
 #include <string.h>
 #include "config.h"
 #include "ps_intern.h"
@@ -85,6 +87,29 @@ ps_bool ps_check_scope(PSDoc *psdoc, int scope) {
 }
 /* }}} */
 
+/* ps_get_resources() {{{
+ * Returns an array of pointers to all resources of a category.
+ */
+PS_RESOURCE **ps_get_resources(PSDoc *psdoc, const char *category, int *count) {
+	PS_CATEGORY *cat;
+	PS_RESOURCE *res;
+	PS_RESOURCE **ress = NULL;
+	int i;
+
+	*count = 0;
+	for(cat = dlst_first(psdoc->categories); cat != NULL; cat = dlst_next(cat)) {
+		if (!strcmp(cat->name, category)) {
+			ress = psdoc->malloc(psdoc, cat->resources->count * sizeof(PS_RESOURCE *), _("Allocate Memory for list of resources."));
+			*count = cat->resources->count;
+			for(i=0, res = dlst_first(cat->resources); res != NULL; res = dlst_next(res), i++) {
+				ress[i] = res;
+			}
+		}
+	}
+	return(ress);
+}
+/* }}} */
+
 /* ps_find_resource() {{{
  */
 char *ps_find_resource(PSDoc *psdoc, const char *category, const char *resource) {
@@ -109,19 +134,24 @@ PS_RESOURCE * ps_add_resource(PSDoc *psdoc, const char *category, const char *re
 	PS_CATEGORY *cat;
 	PS_RESOURCE *res;
 
-	if (strcmp("FontAFM", category) &&
-			strcmp("FontOutline", category) &&
+	/* Right and LeftMarginKerning are set in PS_set_paramter. They manipulate
+	 * the ADOBEINFO struct of the affected char directly.
+	 */
+	if (strcmp("SearchPath", category) &&
+	    strcmp("FontAFM", category) &&
+			strcmp("FontEncoding", category) &&
 			strcmp("FontProtusion", category) &&
-			strcmp("RightMarginKerning", category) &&
-			strcmp("LeftMarginKerning", category)) {
+			strcmp("FontOutline", category)) {
 		return(NULL);
 	}
 
+	/* Search for the category */
 	for(cat = dlst_first(psdoc->categories); cat != NULL; cat = dlst_next(cat)) {
 		if(!strcmp(cat->name, category))
 			break;
 	}
 
+	/* Create a new category if it does not exist */
 	if(cat == NULL) {
 		if(NULL == (cat = (PS_CATEGORY *) dlst_newnode(psdoc->categories, sizeof(PS_CATEGORY)))) {
 			return(NULL);
@@ -131,10 +161,14 @@ PS_RESOURCE * ps_add_resource(PSDoc *psdoc, const char *category, const char *re
 		dlst_insertafter(psdoc->categories, cat, PS_DLST_HEAD(psdoc->categories));
 	}
 
-	// Check if resource already available
-	for(res = dlst_first(cat->resources); res != NULL; res = dlst_next(res)) {
-		if(!strcmp(res->name, resource))
-			break;
+	/* Check if resource already available */
+	if(resource) {
+		for(res = dlst_first(cat->resources); res != NULL; res = dlst_next(res)) {
+			if(!strcmp(res->name, resource))
+				break;
+		}
+	} else {
+		res = NULL;
 	}
 	if(res) {
 		psdoc->free(psdoc, res->name);
@@ -145,7 +179,10 @@ PS_RESOURCE * ps_add_resource(PSDoc *psdoc, const char *category, const char *re
 		if(NULL == (res = (PS_RESOURCE *) dlst_newnode(cat->resources, sizeof(PS_RESOURCE)))) {
 			return(NULL);
 		}
-		res->name = ps_strdup(psdoc, resource);
+		if(resource)
+			res->name = ps_strdup(psdoc, resource);
+		else
+			res->name = NULL;
 		res->value = ps_strdup(psdoc, filename);
 		dlst_insertafter(cat->resources, res, PS_DLST_HEAD(cat->resources));
 	}
@@ -168,8 +205,10 @@ void ps_del_resources(PSDoc *psdoc) {
 		res = dlst_first(cat->resources);
 		while(res != NULL) {
 			nextres = dlst_next(res);
-			psdoc->free(psdoc, res->name);
-			psdoc->free(psdoc, res->value);
+			if(res->name)
+				psdoc->free(psdoc, res->name);
+			if(res->value)
+				psdoc->free(psdoc, res->value);
 			res = nextres;
 		}
 		dlst_kill(cat->resources, dlst_freenode);
@@ -467,7 +506,7 @@ void ps_free_enc_vector(PSDoc *psdoc, ENCODING *enc) {
 unsigned char ps_fontenc_code(ght_hash_table_t *fontenc, char *adobename) {
 	if(fontenc) {
 		int code;
-		code = ght_get(fontenc, strlen(adobename)+1, (void *) adobename);
+		code = (int) ght_get(fontenc, strlen(adobename)+1, (void *) adobename);
 		if(code)
 			return((unsigned char) code -1);
 		else {
@@ -482,9 +521,9 @@ unsigned char ps_fontenc_code(ght_hash_table_t *fontenc, char *adobename) {
 /* ps_inputenc_name() {{{
  * Returns the adobe name for a char in the input encoding vector
  */
-char *ps_inputenc_name(ENCODING *inputenc, unsigned char c) {
-	if(inputenc) {
-		return(inputenc->vec[c]);
+char *ps_inputenc_name(PSDoc *psdoc, unsigned char c) {
+	if(psdoc->inputenc) {
+		return(psdoc->inputenc->vec[c]);
 	} else {
 		return NULL;
 	}
@@ -507,6 +546,273 @@ int ps_get_bool_parameter(PSDoc *psdoc, const char *name, int deflt) {
 }
 /* }}} */
 
+/* ps_open_file_in_path() {{{
+ */
+FILE *ps_open_file_in_path(PSDoc *psdoc, const char *filename) {
+	FILE *fp;
+	PS_RESOURCE **pathres;
+	int count;
+	int i;
+	char buffer[255];
+
+	if(fp = fopen(filename, "r"))
+		return fp;
+
+	if(NULL != (pathres = ps_get_resources(psdoc, "SearchPath", &count))) {
+		for(i=count-1; i>0; i--) {
+			snprintf(buffer, 255, "%s/%s", pathres[i]->value, filename);
+			fprintf(stderr, "Searching for %s\n", buffer);
+			if(fp = fopen(buffer, "r")) {
+				fprintf(stderr, "found %s in %s\n", filename, pathres[i]->value);
+				break;
+			}
+		}
+		psdoc->free(psdoc, pathres);
+		if(fp)
+			return(fp);
+	}
+
+	snprintf(buffer, 255, "%s/%s", PACKAGE_DATA_DIR, filename);
+	if(fp = fopen(buffer, "r"))
+		return fp;
+
+	return NULL;
+}
+/* }}} */
+
+int pow85[5] = {1, 85, 85*85, 85*85*85, 85*85*85*85};
+	
+/* ps_ascii85_encode() {{{
+ */
+int ps_ascii85_encode(PSDoc *psdoc, char *data, size_t len) {
+	unsigned long buffer;
+	unsigned char c;
+	int i, count, cc;
+
+	count = 0;
+	cc = 0;
+	while(count <= len-4) {
+		buffer = ((long) data[count] << 24) + ((long) data[count+1] << 16) + ((long) data[count+2] << 8) + (long) data[count+3];
+		if(buffer == 0) {
+			ps_putc(psdoc, 'z');
+			cc++;
+		} else {
+			for(i=4; i>=0; i--) {
+				c = buffer / pow85[i];
+				ps_putc(psdoc, c+'!');
+				buffer = buffer % pow85[i];
+			}
+			cc += 4;
+		}
+		count += 4;
+		if(cc > 55) {
+			ps_putc(psdoc, '\n');
+			cc = 0;
+		}
+	}
+	if(len-count) {
+		buffer = 0;
+		for(i=0; i<len-count; i++)
+			buffer = (buffer << 8) + (long) data[count+i];
+		buffer = buffer << ((4-len+count)*8);
+		for(i=4; i>=4-len+count; i--) {
+			c = buffer / pow85[i];
+			ps_putc(psdoc, c+'!');
+			buffer = buffer % pow85[i];
+		}
+	}
+	ps_putc(psdoc, '~');
+	ps_putc(psdoc, '>');
+}
+/* }}} */
+
+/* ps_asciihex_encode() {{{
+ */
+int ps_asciihex_encode(PSDoc *psdoc, char *data, size_t len) {
+	int i, cc=0;
+	char *dataptr;
+
+	dataptr = data;
+	for(i=0; i<len; i++) {
+		ps_printf(psdoc, "%02x", *dataptr);
+		dataptr ++;
+		cc++;
+		if(cc > 35) {
+			ps_printf(psdoc, "\n");
+			cc = 0;
+		}
+	}
+	ps_putc(psdoc, '\n');
+	ps_putc(psdoc, '>');
+}
+/* }}} */
+
+/* ps_parse_optlist() {{{
+ */
+ght_hash_table_t *ps_parse_optlist(PSDoc *psdoc, const char *optstr) {
+	int i, isname;
+	char name[100], value[100], delim;
+	const char *optstrptr;
+	ght_hash_table_t *opthash;
+
+	if(optstr == NULL || optstr[0] == '\0')
+		return(NULL);
+
+	opthash = ght_create(30);
+	if(opthash) {
+		ght_set_alloc(opthash, ps_ght_malloc, ps_ght_free, psdoc);
+
+		isname = 1;
+		optstrptr = optstr;
+		name[0] = '\0';
+		value[0] = '\0';
+
+		/* Skip leading spaces */
+		while(*optstrptr == ' ')
+			optstrptr++;
+
+		while(*optstrptr != '\0') {
+			if(isname) {
+				i = 0;
+				while(*optstrptr != '\0' && *optstrptr != ' ') {
+					name[i] = *optstrptr;
+					i++;
+					optstrptr++;
+				}
+				name[i] = '\0';
+				isname = 0;
+				optstrptr++;
+			} else {
+				if(*optstrptr == '{') {
+					delim = '}';
+					optstrptr++;
+				} else {
+					delim = ' ';
+				}
+				i = 0;
+				while(*optstrptr != '\0' && *optstrptr != delim) {
+					value[i] = *optstrptr;
+					i++;
+					optstrptr++;
+				}
+				value[i] = '\0';
+				isname = 1;
+				optstrptr++;
+
+				/* Value and Name is read, insert it into hash */
+				if(name[0]) {
+					ght_insert(opthash, (void *) ps_strdup(psdoc, value), strlen(name)+1, name);
+					name[0] = '\0';
+					value[0] = '\0';
+				}
+			}
+			while(*optstrptr != '\0' && *optstrptr == ' ')
+				optstrptr++;
+		}
+	}
+	return(opthash);
+}
+/* }}} */
+
+/* get_optlist_element_as_float() {{{
+ * Retrieves an element from the hash and converts its value into float
+ * Modifies *value only if the conversion was successful.
+ * Returns -1 if the parameter is not available and -2 if it is available
+ * but the value cannot be retrieved.
+ */
+/* FIXME: For some reason this prototype is needed for strtof() to work */
+float strtof(const char *nptr, char **endptr);
+int get_optlist_element_as_float(PSDoc *psdoc, ght_hash_table_t *opthash, const char *name, float *value) {
+	float tmp;
+	char *strval;
+	char *endval;
+
+	if(NULL == opthash) {
+		return(-1);
+	}
+	if(NULL == (strval = ght_get(opthash, strlen(name)+1, (void *) name))) {
+		return(-1);
+	}
+	tmp = strtof(strval, &endval);
+	if(endval == strval) {
+		return(-2);
+	}
+	*value = tmp;
+	return(0);
+} 
+/* }}} */
+
+/* get_optlist_element_as_int() {{{
+ * Retrieves an element from the hash and converts its value into integer
+ * Modifies *value only if the conversion was successful.
+ * Returns -1 if the parameter is not available and -2 if it is available
+ * but the value cannot be retrieved.
+ */
+int get_optlist_element_as_int(PSDoc *psdoc, ght_hash_table_t *opthash, const char *name, int *value) {
+	long tmp;
+	char *strval;
+	char *endval;
+
+	if(NULL == opthash) {
+		return(-1);
+	}
+	if(NULL == (strval = ght_get(opthash, strlen(name)+1, (void *) name))) {
+		return(-1);
+	}
+	tmp = strtol(strval, &endval, 10);
+	if(endval == strval) {
+		return(-2);
+	}
+	*value = tmp;
+	return(0);
+} 
+/* }}} */
+
+/* get_optlist_element_as_bool() {{{
+ * Retrieves an element from the hash and converts its value into boolean
+ * Returns -1 if the parameter is not available and -2 if it is available
+ * but the value cannot be retrieved.
+ */
+int get_optlist_element_as_bool(PSDoc *psdoc, ght_hash_table_t *opthash, const char *name, ps_bool *value) {
+	char *strval;
+
+	if(NULL == opthash) {
+		return(-1);
+	}
+	if(NULL == (strval = ght_get(opthash, strlen(name)+1, (void *) name))) {
+		return(-1);
+	}
+	if(strcmp(strval, "false") == 0) {
+		*value = ps_false;
+	} else if(strcmp(strval, "true") == 0) {
+		*value = ps_true;
+	} else {
+		return(-2);
+	}
+	return(0);
+} 
+/* }}} */
+
+/* get_optlist_element_as_string() {{{
+ * Retrieves an element from the hash and converts its value into string
+ * Returns -1 if the parameter is not available
+ */
+int get_optlist_element_as_string(PSDoc *psdoc, ght_hash_table_t *opthash, const char *name, char **value) {
+	char *strval;
+
+	if(NULL == opthash) {
+		return(-1);
+	}
+	if(NULL == (strval = ght_get(opthash, strlen(name)+1, (void *) name))) {
+		return(-1);
+	}
+
+	*value = strval;
+
+	return(0);
+} 
+/* }}} */
+
 /*
  * Local variables:
  * tab-width: 4
